@@ -111,7 +111,10 @@ NB_MODULE(MODULE_NAME, m) {
             :param delay_den: The delay denominator for this frame (defaults to DEFAULT_FRAME_DENMINATOR)
 
             :return: A APNGFrame object.
+            :rtype: apngasm_python._apngasm_python.APNGFrame
         )pbdoc");
+    
+    nb::class_<apngasm::listener::IAPNGAsmListener>(m, "IAPNGAsmListener");
 
     nb::class_<apngasm::rgb>(m, "rgb")
         .def(nb::init<>(),
@@ -430,8 +433,9 @@ NB_MODULE(MODULE_NAME, m) {
             Disassembles an APNG file.
             
             :param str file_path: The file path to the PNG image to be disassembled.
+
             :return: A vector containing the frames of the disassembled PNG.
-            :rtype: list")
+            :rtype: list
         )pbdoc")
 
         .def("save_pngs", &apngasm::APNGAsm::savePNGs,
@@ -453,7 +457,10 @@ NB_MODULE(MODULE_NAME, m) {
             For more details on animation specs see:
             https://github.com/Genshin/PhantomStandards
             
-            :param str file_path: The path of JSON file
+            :param str file_path: The path of JSON or XML file
+
+            :return: A vector containing the frames
+            :rtype: list
         )pbdoc")
 
         .def("save_json", &apngasm::APNGAsm::saveJSON,
@@ -463,6 +470,7 @@ NB_MODULE(MODULE_NAME, m) {
             
             :param str output_path: Path to save the file to.
             :param str image_dir: Directory where frame files are to be saved if not the same path as the animation spec.
+
             :return: true if save was successful.
             :rtype: bool
         )pbdoc")
@@ -484,7 +492,7 @@ NB_MODULE(MODULE_NAME, m) {
         R"pbdoc(
             Sets a listener.
             
-            :param listener: A pointer to the listener object. If the argument is NULL a default APNGAsmListener will be created and assigned.
+            :param listener: A pointer to the listener object. If the argument is NULL a default APNGAsmListener will be created and assigned.8
         )pbdoc")
 
         .def("set_loops", &apngasm::APNGAsm::setLoops,
@@ -506,6 +514,9 @@ NB_MODULE(MODULE_NAME, m) {
         .def("get_frames", &apngasm::APNGAsm::getFrames,
         R"pbdoc(
             Returns the frame vector.
+
+            :return: frame vector
+            :rtype: numpy.ndarray
         )pbdoc")
 
         .def("get_loops", &apngasm::APNGAsm::getLoops,
